import { useState, useRef, useEffect } from 'react'
import type { ChecklistItem as ChecklistItemType } from '../types'
import DependencyManager from './DependencyManager'
import { DateConstraintManager } from './DateConstraintManager'
import { Checkbox, Flex, Box, Text, IconButton, Badge, Tooltip } from '@radix-ui/themes'
import { LockClosedIcon, TrashIcon, ClockIcon, LinkBreak2Icon } from '@radix-ui/react-icons'

interface ChecklistItemProps {
  item: ChecklistItemType
  onUpdate: (id: number, updates: Partial<ChecklistItemType>) => Promise<void>
  onDelete: (id: number) => Promise<void>
  onLock: (id: number, user: string) => Promise<void>
  depth?: number
  children?: ChecklistItemType[]
  allItems?: ChecklistItemType[]
}

export default function ChecklistItem({ 
  item, 
  onUpdate, 
  onDelete, 
  onLock, 
  depth = 0,
  children = [],
  allItems = []
}: ChecklistItemProps) {
  const [isEditing, setIsEditing] = useState(false)
  const [isDeleting, setIsDeleting] = useState(false)
  const [isDependencyModalOpen, setIsDependencyModalOpen] = useState(false)
  const [isDateConstraintModalOpen, setIsDateConstraintModalOpen] = useState(false)
  const [userName, setUserName] = useState('')
  const contentRef = useRef<HTMLDivElement>(null)

  const isLocked = item.locked_by && item.lock_until && new Date(item.lock_until) > new Date()
  const isLockedByMe = isLocked && item.locked_by === userName

  // Check if all dependencies are completed
  const dependenciesCompleted = item.dependencies?.every(depId => {
    const depItem = allItems.find(i => i.id === depId)
    return depItem?.checked
  }) ?? true

  // Check date constraints
  const now = new Date()
  const notBeforeDate = item.not_before ? new Date(item.not_before) : null
  const notAfterDate = item.not_after ? new Date(item.not_after) : null
  const dateConstraintsMet = (!notBeforeDate || now >= notBeforeDate) && (!notAfterDate || now <= notAfterDate)

  // Check if item can be completed (all dependencies met and date constraints met)
  const canComplete = (dependenciesCompleted && dateConstraintsMet) || item.checked

  // Get dependency items for display
  const dependencyItems = item.dependencies?.map(depId => 
    allItems.find(i => i.id === depId)
  ).filter(Boolean) ?? []

  useEffect(() => {
    // Generate a random user name if not set
    if (!userName) {
      const randomName = `user_${Math.random().toString(36).substr(2, 9)}`
      setUserName(randomName)
    }
  }, [userName])

  useEffect(() => {
    if (isEditing && contentRef.current) {
      contentRef.current.focus()
      // Select all text when entering edit mode
      const range = document.createRange()
      range.selectNodeContents(contentRef.current)
      const selection = window.getSelection()
      selection?.removeAllRanges()
      selection?.addRange(range)
    }
  }, [isEditing])

  const handleEdit = async () => {
    if (isLocked && !isLockedByMe) {
      alert('This item is being edited by someone else')
      return
    }

    if (!isLockedByMe) {
      try {
        await onLock(item.id, userName)
      } catch (error) {
        console.error('Failed to lock item:', error)
        return
      }
    }

    setIsEditing(true)
  }

  const handleSave = async () => {
    const newContent = contentRef.current?.textContent?.trim() || ''
    if (newContent === '') return

    try {
      await onUpdate(item.id, { content: newContent })
      setIsEditing(false)
    } catch (error) {
      console.error('Failed to update item:', error)
    }
  }

  const handleCancel = () => {
    if (contentRef.current) {
      contentRef.current.textContent = item.content
    }
    setIsEditing(false)
  }

  const handleDelete = async () => {
    if (confirm('Are you sure you want to delete this item?')) {
      try {
        setIsDeleting(true)
        await onDelete(item.id)
      } catch (error) {
        console.error('Failed to delete item:', error)
      } finally {
        setIsDeleting(false)
      }
    }
  }

  const handleToggleCheck = async () => {
    // Don't allow unchecking if already checked
    if (item.checked) {
      try {
        await onUpdate(item.id, { checked: false })
      } catch (error) {
        console.error('Failed to uncheck item:', error)
      }
      return
    }

    // Check if dependencies are met before allowing completion
    if (!dependenciesCompleted) {
      alert(`Cannot complete this item. The following dependencies must be completed first:\n\n${dependencyItems.map(dep => `• ${dep?.content}`).join('\n')}`)
      return
    }

    // Check if date constraints are met
    if (!dateConstraintsMet) {
      let message = 'Cannot complete this item due to date constraints:\n\n'
      if (notBeforeDate && now < notBeforeDate) {
        message += `• Cannot complete before ${notBeforeDate.toLocaleString()}\n`
      }
      if (notAfterDate && now > notAfterDate) {
        message += `• Cannot complete after ${notAfterDate.toLocaleString()}\n`
      }
      alert(message)
      return
    }

    try {
      await onUpdate(item.id, { checked: true })
    } catch (error) {
      console.error('Failed to toggle item:', error)
      if (error instanceof Error && error.message.includes('cannot complete item:')) {
        alert(error.message)
      }
    }
  }

  const handleKeyDown = (e: React.KeyboardEvent) => {
    if (e.key === 'Enter') {
      e.preventDefault()
      handleSave()
    } else if (e.key === 'Escape') {
      e.preventDefault()
      handleCancel()
    }
  }

  const handleBlur = () => {
    // Small delay to allow for button clicks
    setTimeout(() => {
      if (isEditing) {
        handleSave()
      }
    }, 100)
  }

  const handleDateConstraintSave = async (notBefore?: string, notAfter?: string) => {
    try {
      await onUpdate(item.id, { not_before: notBefore, not_after: notAfter })
      setIsDateConstraintModalOpen(false)
    } catch (error) {
      console.error('Failed to update date constraints:', error)
    }
  }

  return (
    <Box asChild>
      <li style={{ listStyle: 'none' }}>
        <Flex 
          gap="3" 
          align="start" 
          p="3" 
          className="group rounded-lg hover:bg-gray-50 dark:hover:bg-gray-800/50 transition-all duration-200 border border-transparent hover:border-gray-200 dark:hover:border-gray-700"
        >
          <Box mt="1">
            <Checkbox
              checked={item.checked}
              onCheckedChange={handleToggleCheck}
              disabled={Boolean(isLocked && !isLockedByMe) || (!item.checked && !canComplete)}
              size="2"
            />
          </Box>
        
          <Box style={{ flex: 1, minWidth: 0 }}>
            {isEditing ? (
              <Box
                ref={contentRef}
                contentEditable={true}
                suppressContentEditableWarning={true}
                onKeyDown={handleKeyDown}
                onBlur={handleBlur}
                className="px-3 py-2 border-2 border-blue-500 rounded-md focus:ring-2 focus:ring-blue-200 dark:focus:ring-blue-800 bg-white dark:bg-gray-800 shadow-sm outline-none"
                style={{ minHeight: '28px' }}
              >
                {item.content}
              </Box>
            ) : (
              <Text
                size="2"
                onClick={handleEdit}
                className={`cursor-pointer hover:text-blue-600 dark:hover:text-blue-400 ${
                  item.checked ? 'line-through opacity-75' : ''
                } ${
                  !item.checked && !canComplete ? 'opacity-60' : ''
                }`}
                color={item.checked ? "gray" : undefined}
              >
                {item.content}
              </Text>
            )}
          
            {/* Consolidated status indicators - only show when item cannot be completed */}
            {!item.checked && !canComplete && (
              <Flex gap="1" mt="1" align="center">
                {/* Single consolidated status indicator */}
                <Badge 
                  color="orange" 
                  variant="soft" 
                  size="1"
                  className="flex items-center gap-1 opacity-80"
                >
                  <span className="text-xs">⚠️</span>
                  <span className="text-xs font-medium">
                    {dependencyItems.length > 0 && !dateConstraintsMet 
                      ? `${dependencyItems.length} deps + time locked`
                      : dependencyItems.length > 0 
                      ? `${dependencyItems.length} deps`
                      : 'Time locked'
                    }
                  </span>
                </Badge>
              </Flex>
            )}
          </Box>

          <Flex gap="2" align="center" className="sm:opacity-0 sm:group-hover:opacity-100 transition-opacity duration-200">
            {/* Only show essential status indicators in hover area */}
            {!item.checked && !canComplete && (
              <Tooltip content={
                <Box p="3" className="max-w-xs">
                  <Flex direction="column" gap="2">
                    <Text size="2" weight="medium" color="gray">
                      Cannot complete this item:
                    </Text>
                    {dependencyItems.length > 0 && (
                      <Flex gap="2" align="center">
                        <span className="text-orange-500">🔗</span>
                        <Text size="1" color="orange">
                          Depends on: {dependencyItems.map(dep => dep?.content).join(', ')}
                        </Text>
                      </Flex>
                    )}
                    {!dateConstraintsMet && (
                      <Flex gap="2" align="center">
                        <span className="text-red-500">🕐</span>
                        <Text size="1" color="red">
                          {notBeforeDate && now < notBeforeDate 
                            ? `Cannot complete before ${notBeforeDate.toLocaleString()}`
                            : notAfterDate && now > notAfterDate
                            ? `Cannot complete after ${notAfterDate.toLocaleString()}`
                            : 'Date constraint not met'
                          }
                        </Text>
                      </Flex>
                    )}
                  </Flex>
                </Box>
              }>
                <Badge 
                  color="orange" 
                  variant="soft" 
                  size="1"
                  className="cursor-help hover:opacity-80 transition-opacity"
                >
                  <span className="text-xs">⚠️</span>
                </Badge>
              </Tooltip>
            )}
            
            {isLocked && !isLockedByMe && (
              <Badge color="red" variant="surface" size="1">
                <LockClosedIcon width="12" height="12" />
                {item.locked_by}
              </Badge>
            )}
            
            {!isEditing && (
              <>
                <Tooltip content="Manage dependencies">
                  <IconButton 
                    size="1" 
                    variant="ghost" 
                    onClick={() => setIsDependencyModalOpen(true)}
                  >
                    <LinkBreak2Icon width="16" height="16" />
                  </IconButton>
                </Tooltip>
                <Tooltip content="Manage date constraints">
                  <IconButton 
                    size="1" 
                    variant="ghost" 
                    onClick={() => setIsDateConstraintModalOpen(true)}
                  >
                    <ClockIcon width="16" height="16" />
                  </IconButton>
                </Tooltip>
                <Tooltip content="Delete item">
                  <IconButton 
                    size="1" 
                    variant="ghost" 
                    color="red"
                    onClick={handleDelete}
                    disabled={isDeleting}
                  >
                    <TrashIcon width="16" height="16" />
                  </IconButton>
                </Tooltip>
              </>
            )}
          </Flex>
        </Flex>

        {children.length > 0 && (
          <Box ml="8" mt="2" pl="4" className="space-y-2 border-l-2 border-gray-200 dark:border-gray-700">
            {children.map(child => (
              <ChecklistItem
                key={child.id}
                item={child}
                onUpdate={onUpdate}
                onDelete={onDelete}
                onLock={onLock}
                depth={depth + 1}
                allItems={allItems}
              />
            ))}
          </Box>
        )}

      {/* Dependency Manager Modal */}
      <DependencyManager
        item={item}
        allItems={allItems}
        onUpdate={onUpdate}
        onClose={() => setIsDependencyModalOpen(false)}
        isOpen={isDependencyModalOpen}
      />

      {/* Date Constraint Manager Modal */}
      {isDateConstraintModalOpen && (
        <DateConstraintManager
          item={item}
          onSave={handleDateConstraintSave}
          onCancel={() => setIsDateConstraintModalOpen(false)}
        />
      )}
      </li>
    </Box>
  )
} 