import React, { useState, useEffect } from 'react'
import type { ChecklistItem } from '../types'
import { Dialog, Flex, Text, Button, Heading, Card, Box, IconButton } from '@radix-ui/themes'
import { Cross2Icon, CalendarIcon, ClockIcon } from '@radix-ui/react-icons'

interface DateConstraintManagerProps {
  item: ChecklistItem
  onSave: (notBefore?: string, notAfter?: string) => void
  onCancel: () => void
}

export const DateConstraintManager: React.FC<DateConstraintManagerProps> = ({
  item,
  onSave,
  onCancel
}) => {
  // Parse existing dates to local datetime format for inputs
  const formatDateTimeLocal = (date: Date | null): string => {
    if (!date) return ''
    const year = date.getFullYear()
    const month = String(date.getMonth() + 1).padStart(2, '0')
    const day = String(date.getDate()).padStart(2, '0')
    const hours = String(date.getHours()).padStart(2, '0')
    const minutes = String(date.getMinutes()).padStart(2, '0')
    return `${year}-${month}-${day}T${hours}:${minutes}`
  }

  const [notBeforeValue, setNotBeforeValue] = useState<string>(
    item.not_before ? formatDateTimeLocal(new Date(item.not_before)) : ''
  )
  const [notAfterValue, setNotAfterValue] = useState<string>(
    item.not_after ? formatDateTimeLocal(new Date(item.not_after)) : ''
  )

  // Get current datetime in local format for min attribute
  const now = new Date()
  const minDateTime = formatDateTimeLocal(now)

  const handleSave = () => {
    const notBeforeDate = notBeforeValue ? new Date(notBeforeValue).toISOString() : undefined
    const notAfterDate = notAfterValue ? new Date(notAfterValue).toISOString() : undefined
    onSave(notBeforeDate, notAfterDate)
  }

  const handleClear = () => {
    setNotBeforeValue('')
    setNotAfterValue('')
  }

  // Update min value for notAfter when notBefore changes
  useEffect(() => {
    if (notBeforeValue && notAfterValue) {
      const notBeforeDate = new Date(notBeforeValue)
      const notAfterDate = new Date(notAfterValue)
      if (notBeforeDate > notAfterDate) {
        setNotAfterValue(notBeforeValue)
      }
    }
  }, [notBeforeValue, notAfterValue])

  return (
    <Dialog.Root open={true} onOpenChange={onCancel}>
      <Dialog.Content style={{ maxWidth: 450 }}>
        <Dialog.Title>Manage Date Constraints</Dialog.Title>
        <Dialog.Description size="2" mb="4">
          Set when this item can be completed. Leave empty to remove constraints.
        </Dialog.Description>
        
        <Flex direction="column" gap="4">
          <Box>
            <Flex align="center" gap="2" mb="2">
              <CalendarIcon />
              <Text size="2" weight="medium">
                Not Before (Earliest completion time)
              </Text>
            </Flex>
            <Box position="relative">
              <input
                type="datetime-local"
                value={notBeforeValue}
                onChange={(e) => setNotBeforeValue(e.target.value)}
                min={minDateTime}
                className="w-full px-3 py-2 border border-gray-300 dark:border-gray-600 rounded-md focus:outline-none focus:ring-2 focus:ring-blue-500 focus:border-transparent bg-white dark:bg-gray-700 text-gray-900 dark:text-white"
                style={{ 
                  colorScheme: 'dark',
                  WebkitAppearance: 'none',
                  MozAppearance: 'none'
                }}
              />
              {notBeforeValue && (
                <IconButton
                  size="1"
                  variant="ghost"
                  color="gray"
                  onClick={() => setNotBeforeValue('')}
                  style={{
                    position: 'absolute',
                    right: '8px',
                    top: '50%',
                    transform: 'translateY(-50%)'
                  }}
                >
                  <Cross2Icon />
                </IconButton>
              )}
            </Box>
          </Box>
          
          <Box>
            <Flex align="center" gap="2" mb="2">
              <ClockIcon />
              <Text size="2" weight="medium">
                Not After (Latest completion time)
              </Text>
            </Flex>
            <Box position="relative">
              <input
                type="datetime-local"
                value={notAfterValue}
                onChange={(e) => setNotAfterValue(e.target.value)}
                min={notBeforeValue || minDateTime}
                className="w-full px-3 py-2 border border-gray-300 dark:border-gray-600 rounded-md focus:outline-none focus:ring-2 focus:ring-blue-500 focus:border-transparent bg-white dark:bg-gray-700 text-gray-900 dark:text-white"
                style={{ 
                  colorScheme: 'dark',
                  WebkitAppearance: 'none',
                  MozAppearance: 'none'
                }}
              />
              {notAfterValue && (
                <IconButton
                  size="1"
                  variant="ghost"
                  color="gray"
                  onClick={() => setNotAfterValue('')}
                  style={{
                    position: 'absolute',
                    right: '8px',
                    top: '50%',
                    transform: 'translateY(-50%)'
                  }}
                >
                  <Cross2Icon />
                </IconButton>
              )}
            </Box>
          </Box>

          {/* Quick select buttons for common time periods */}
          <Box>
            <Text size="2" weight="medium" mb="2">
              Quick select:
            </Text>
            <Flex gap="2" wrap="wrap">
              <Button 
                size="1" 
                variant="soft"
                onClick={() => {
                  const tomorrow = new Date()
                  tomorrow.setDate(tomorrow.getDate() + 1)
                  tomorrow.setHours(9, 0, 0, 0)
                  setNotBeforeValue(formatDateTimeLocal(tomorrow))
                }}
              >
                Tomorrow 9AM
              </Button>
              <Button 
                size="1" 
                variant="soft"
                onClick={() => {
                  const nextWeek = new Date()
                  nextWeek.setDate(nextWeek.getDate() + 7)
                  nextWeek.setHours(9, 0, 0, 0)
                  setNotBeforeValue(formatDateTimeLocal(nextWeek))
                }}
              >
                Next week
              </Button>
              <Button 
                size="1" 
                variant="soft"
                onClick={() => {
                  const endOfWeek = new Date()
                  const daysUntilFriday = (5 - endOfWeek.getDay() + 7) % 7 || 7
                  endOfWeek.setDate(endOfWeek.getDate() + daysUntilFriday)
                  endOfWeek.setHours(17, 0, 0, 0)
                  setNotAfterValue(formatDateTimeLocal(endOfWeek))
                }}
              >
                End of week
              </Button>
            </Flex>
          </Box>
        </Flex>

        {(notBeforeValue || notAfterValue) && (
          <Card size="2" mt="4">
            <Heading size="2" mb="2">Preview:</Heading>
            <Flex direction="column" gap="1">
              {notBeforeValue && (
                <Text size="2" color="gray">✅ Can complete after: {new Date(notBeforeValue).toLocaleString()}</Text>
              )}
              {notAfterValue && (
                <Text size="2" color="gray">✅ Can complete before: {new Date(notAfterValue).toLocaleString()}</Text>
              )}
              {notBeforeValue && notAfterValue && (
                <Text size="2" color="blue" weight="medium">
                  Time window: {Math.round((new Date(notAfterValue).getTime() - new Date(notBeforeValue).getTime()) / (1000 * 60 * 60 * 24))} days
                </Text>
              )}
            </Flex>
          </Card>
        )}

        <Flex gap="3" mt="4" justify="between">
          <Button variant="ghost" onClick={handleClear}>
            Clear All
          </Button>
          <Flex gap="2">
            <Dialog.Close>
              <Button variant="soft" color="gray">
                Cancel
              </Button>
            </Dialog.Close>
            <Button onClick={handleSave}>
              Save
            </Button>
          </Flex>
        </Flex>
      </Dialog.Content>
    </Dialog.Root>
  )
} 