import { useState, useEffect } from 'react'
import type { ChecklistItem } from '../types'
import { Dialog, Flex, Text, Button, Checkbox, ScrollArea, Badge, Separator } from '@radix-ui/themes'

interface DependencyManagerProps {
  item: ChecklistItem
  allItems: ChecklistItem[]
  onUpdate: (id: number, updates: Partial<ChecklistItem>) => Promise<void>
  onClose: () => void
  isOpen: boolean
}

export default function DependencyManager({ 
  item, 
  allItems, 
  onUpdate, 
  onClose, 
  isOpen 
}: DependencyManagerProps) {
  const [selectedDependencies, setSelectedDependencies] = useState<number[]>([])
  const [isUpdating, setIsUpdating] = useState(false)

  useEffect(() => {
    if (isOpen) {
      setSelectedDependencies(item.dependencies || [])
    }
  }, [isOpen, item.dependencies])

  const handleSave = async () => {
    setIsUpdating(true)
    try {
      await onUpdate(item.id, { dependencies: selectedDependencies })
      onClose()
    } catch (error) {
      console.error('Failed to update dependencies:', error)
    } finally {
      setIsUpdating(false)
    }
  }

  const toggleDependency = (depId: number) => {
    setSelectedDependencies(prev => 
      prev.includes(depId) 
        ? prev.filter(id => id !== depId)
        : [...prev, depId]
    )
  }

  const availableItems = allItems.filter(otherItem => 
    otherItem.id !== item.id && 
    !otherItem.dependencies?.includes(item.id) // Prevent circular dependencies
  )

  return (
    <Dialog.Root open={isOpen} onOpenChange={onClose}>
      <Dialog.Content style={{ maxWidth: 450 }}>
        <Dialog.Title>Manage Dependencies</Dialog.Title>
        <Dialog.Description size="2" mb="4">
          Select items that must be completed before "{item.content}" can be completed:
        </Dialog.Description>

        <ScrollArea type="auto" scrollbars="vertical" style={{ height: 300 }}>
          {availableItems.length === 0 ? (
            <Flex align="center" justify="center" p="4">
              <Text color="gray">No available items to depend on</Text>
            </Flex>
          ) : (
            <Flex direction="column" gap="1">
              {availableItems.map((otherItem, index) => (
                <div key={otherItem.id}>
                  <Flex 
                    align="center" 
                    gap="3" 
                    p="3"
                    className="hover:bg-gray-50 dark:hover:bg-gray-800 cursor-pointer rounded-md"
                    onClick={() => toggleDependency(otherItem.id)}
                  >
                    <Checkbox
                      checked={selectedDependencies.includes(otherItem.id)}
                      onCheckedChange={() => toggleDependency(otherItem.id)}
                    />
                    <Text 
                      size="2" 
                      style={{ 
                        textDecoration: otherItem.checked ? 'line-through' : 'none',
                        opacity: otherItem.checked ? 0.6 : 1,
                        flex: 1
                      }}
                    >
                      {otherItem.content}
                    </Text>
                    {otherItem.checked && (
                      <Badge color="green" size="1">
                        ✓ Done
                      </Badge>
                    )}
                  </Flex>
                  {index < availableItems.length - 1 && <Separator size="4" />}
                </div>
              ))}
            </Flex>
          )}
        </ScrollArea>

        <Flex gap="3" mt="4" justify="end">
          <Dialog.Close>
            <Button variant="soft" color="gray">
              Cancel
            </Button>
          </Dialog.Close>
          <Button onClick={handleSave} disabled={isUpdating}>
            {isUpdating ? 'Saving...' : 'Save'}
          </Button>
        </Flex>
      </Dialog.Content>
    </Dialog.Root>
  )
} 